"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const gray_matter_1 = __importDefault(require("gray-matter"));
const lodash_isempty_1 = __importDefault(require("lodash.isempty"));
const path_1 = __importDefault(require("path"));
const tree_1 = require("./tree");
function resolveRoutePaths(tree, importPrefix, readFile) {
    return tree_1.map(tree, (file) => {
        const segments = file.path.split('/');
        const meta = {
            name: pathToName(segments),
            path: pathToRoute(segments, 0, false),
            pathSegments: toActualPath(segments),
            component: null,
            specifier: pathToSpecifier(segments)
        };
        if (file.type === 'file') {
            const content = readFile(file.path);
            const { data } = gray_matter_1.default(content);
            if (!lodash_isempty_1.default(data)) {
                meta.routeMeta = data;
            }
            meta.component = importPrefix + file.path;
        }
        return meta;
    });
}
exports.resolveRoutePaths = resolveRoutePaths;
function isOmittable(segment) {
    return segment === 'index';
}
/**
 * - Remove `.md` from the last path
 * - Omit if the last segment is `index`
 * - Convert dynamic route to `:param` format
 */
function toActualPath(segments) {
    const lastIndex = segments.length - 1;
    const last = basename(segments[lastIndex]);
    const segmentsWithoutLast = segments.slice(0, -1);
    return isOmittable(last)
        ? segmentsWithoutLast
        : segmentsWithoutLast.concat(last);
}
function pathToName(segments) {
    const last = segments[segments.length - 1];
    return segments
        .slice(0, -1)
        .concat(basename(last))
        .join('-');
}
function pathToSpecifier(segments) {
    const name = pathToName(segments);
    const replaced = name
        .replace(/(^|[^a-zA-Z])([a-zA-Z])/g, (_, a, b) => {
        return a + b.toUpperCase();
    })
        .replace(/[^a-zA-Z0-9]/g, '');
    return /^\d/.test(replaced) ? '_' + replaced : replaced;
}
function pathToRoute(segments, parentDepth, nested) {
    const prefix = nested || parentDepth > 0 ? '' : '/';
    return (prefix +
        toActualPath(segments)
            .slice(parentDepth)
            .join('/'));
}
function basename(filename) {
    return path_1.default.basename(filename, '.md');
}
